@@ -11,11 +11,11 @@ import wsgiref.validate
 from wsgiref.simple_server import make_server
 try:
     from http.cookies import BaseCookie
-    from urllib.parse import unquote, urlparse, urlunparse
+    from urllib.parse import quote, unquote, urlparse, urlunparse
 except ImportError:
     from Cookie import BaseCookie  # NOQA
     from urlparse import urlparse, urlunparse  # NOQA
-    from urllib import unquote  # NOQA
+    from urllib import quote, unquote  # NOQA
 
 from functools import wraps
 from itertools import chain
@@ -31,7 +31,7 @@ from pesto.request import Request
 from pesto.wsgiutils import uri_join, make_query
 from pesto.httputils import parse_header
 
-__version__ = '7.3.5'
+__version__ = '7.3.6'
 
 #: Registry for xpath multimethods
 xpath_registry = {}
@@ -1668,12 +1668,13 @@ class TestAgent(object):
         host = self.environ_defaults['SERVER_NAME']
         port = int(self.environ_defaults['SERVER_PORT'])
 
-        server = make_server(host, port, PassStateWSGIApp(self))
-        url = self.request.make_url(netloc='{0}:{1}'.format(host, port))
+        url = self.request.make_uri(netloc='{0}:{1}'.format(host, port))
+        server = make_server(host, port,
+                             PassStateWSGIApp(self, self.request.request_path))
         if open_in_browser:
             webbrowser.open_new_tab(url)
-        print("\nStarting HTTP server on {0}:{1}. "
-              "Press ctrl-c to exit.".format(host, port))
+        print("\nStarting HTTP server on {0}\n"
+              "Press ctrl-c to exit.".format(url))
         try:
             server.serve_forever()
         except KeyboardInterrupt:
@@ -1702,27 +1703,45 @@ class PassStateWSGIApp(object):
     Used by ``TestAgent.serve``.
     """
 
-    def __init__(self, testagent):
+    def __init__(self, testagent, initial_path):
         self.first_request_served = False
         self.testagent = testagent
+        self.initial_path = initial_path
 
     def __call__(self, environ, start_response):
         if self.first_request_served:
             return self.testagent.validated_app(environ, start_response)
-        self.first_request_served = True
+        if Request(environ).request_path != self.initial_path:
+            return self.redirect_to_initial_path(environ, start_response)
         return self.first_request(environ, start_response)
 
+    def redirect_to_initial_path(self, environ, start_response):
+        start_response('302 Found', [('Content-Type', 'text/html'),
+                                     ('Location', self.initial_path)])
+        return ['<a href="{0}">{0}</a>'.format(self.initial_path)
+                                       .encode('utf8')]
+
     def first_request(self, environ, start_response):
+        self.first_request_served = True
         response = self.testagent.response
         mimetype, charset = parse_header(response.get_header('Content-Type'))
         charset = dict(charset).get('charset', 'UTF-8')
         self.testagent._read_response()
-        if is_html(response) and self.testagent._lxml is not None:
-            response = response.replace(content=[tostring(self.testagent.lxml,
-                                                          encoding=charset)])
-        else:
-            response = response.replace(content=[self.testagent._body])
 
+        if is_html(response) and self.testagent._lxml is not None:
+            content = tostring(self.testagent.lxml, encoding=charset)
+        else:
+            content = self.testagent._body
+        assert isinstance(content, bytes)
+
+        if is_html(response):
+            original_url = base_url(self.testagent.request.environ)
+            served_url = base_url(environ)
+            if original_url != served_url:
+                content = content.replace(original_url.encode(charset),
+                                          served_url.encode(charset))
+
+        response = response.replace(content=[content])
         for key, morsel in self.testagent.cookies.items():
             response = response.add_header('Set-Cookie', morsel.OutputString())
         return response(environ, start_response)
@@ -1814,3 +1833,25 @@ def unicode_to_environ(s, enc='UTF-8'):
     This implementation good for python2 only
     """
     return s.encode(enc, 'surrogateescape')
+
+
+def base_url(environ):
+    """
+    Return the base URL for the request (ie everything up to SCRIPT_NAME;
+    PATH_INFO and QUERY_STRING are not included)
+    """
+    url = environ['wsgi.url_scheme'] + '://'
+
+    if environ.get('HTTP_HOST'):
+        url += environ['HTTP_HOST']
+    else:
+        url += environ['SERVER_NAME']
+
+        if environ['wsgi.url_scheme'] == 'https':
+            if environ['SERVER_PORT'] != '443':
+                url += ':' + environ['SERVER_PORT']
+        elif environ['SERVER_PORT'] != '80':
+                url += ':' + environ['SERVER_PORT']
+
+    url += quote(environ.get('SCRIPT_NAME', ''))
+    return url
